task ballgown {
  File outdir
  File dir
  String vs
  Float logfc
  Float fdr  
  Array[File] new
  command {
	 Rscript  /bio/ballgown.r --inputdir  ${dir}  --outdir ${outdir}  --name  ${vs}  --fdr ${fdr} --logfc ${logfc}
  }
  output {
        File diff = "${outdir}/${vs}.ballgown_gene_all.xls"
        File deg = "${outdir}/${vs}.ballgown_gene_deg.xls"
  }
  runtime{
     docker:"ballgown:latest"
     cpu:"1"
     memory:"5GB"
  }
}
task deseq2 {
  File outdir
  File count
  String vs
  Float logfc
  Float fdr 

  command {
	 Rscript  /bio/DESeq2.r  --count  ${count}  --outdir ${outdir}  --name  ${vs} --fdr ${fdr} --logfc ${logfc} 
  }
  output {
        File diff = "${outdir}/${vs}.DESeq2_all.xls"
        File deg = "${outdir}/${vs}.DESeq2_deg.xls"
  }
  runtime{
     docker:"deseq2:latest"
     cpu:"1"
     memory:"5GB"
  }
}

task edger {

  File outdir
  File count
  String vs
  Float logfc
  Float fdr 

  command {
	 Rscript  /bio/edgeR.r  --count  ${count}  --outdir ${outdir}  --name  ${vs} --fdr ${fdr} --logfc ${logfc} 
  }
  output {
        File diff = "${outdir}/${vs}_edgeR_all.xls"
        File deg = "${outdir}/${vs}_edgeR_deg.xls"
  }
  runtime{
     docker:"edger:latest"
     cpu:"1"
     memory:"5GB"
  }
}

task volcano {

  File all
  File outdir
  String vs
  Float logfc
  Float fdr 

  command {
	 Rscript  /script/plot_volcano.r  --input   ${all}  --outdir ${outdir}  --name  ${vs} --fdr ${fdr} --logfc ${logfc} 
  }
  runtime{
     docker:"ggplot2:latest"
     cpu:"1"
     memory:"5GB"
  }
}


task fpkm_exact {

  File deg
  String group
  File fpkm

  command {
	perl /opt/deg_abstract_fpkm.pl -inputs ${fpkm} -out  ${group}.fpkm.txt -group  ${group} -deg ${deg} 
  }
  output {
        File out = "${group}.fpkm.txt"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"1GB"
  }
}

task heatmap_plot {

  File fpkm 
  String group
  File outdir
  String prefix
  command {
	line=`cat ${fpkm} | wc -l`
	threshold="10"
	if  [ $line -gt $threshold ] 
	then 
		 Rscript  /opt/heatmaply_public.r --infile ${fpkm} --prefix ${group}.${prefix} --outdir ${outdir}
	fi
  }
  output {
        File? html = "${outdir}/${group}.${prefix}.heatmap.html"
  }
  runtime{
     docker:"heatmap:latest"
     cpu:"1"
     memory:"5GB"
  }
}
