task annotate {
  File deg
  File all
  String idtype
  String species
  File kobasdb
  String enrich

  command <<<
	 line=`cat ${deg} | wc -l`
	 threshold="2"
	 if  [ $line -gt $threshold ] 
	 then 
		export PYTHONPATH=/BioBin/kobas-2.1.1_20160822/src
		python /BioBin/kobas-2.1.1_20160822/scripts/annotate.py -i ${deg} -t ${idtype} -s ${species} -n 25 -o  ${deg}.anno.txt  -q ${kobasdb}
		python /BioBin/kobas-2.1.1_20160822/scripts/annotate.py -i ${all} -t ${idtype} -s ${species} -n 25 -o  ${all}.anno.txt  -q ${kobasdb}
		python /BioBin/kobas-2.1.1_20160822/scripts/identify.py  -f  ${deg}.anno.txt  -b ${all}.anno.txt  -d  ${enrich}  -o  ${deg}.enrich.xls -q  ${kobasdb}


	fi
  >>>
  output {
        File?   danno = "${deg}.anno.txt" 
        File?   eanno = "${all}.anno.txt" 
	File?   kegg= "${deg}.enrich.xls"
  }
  runtime{
     docker:"kobas:latest"
     cpu:"15"
     memory:"30GB"
  }
}


task list {
  File deg
  File all
  File realtion
  String  group
  File dir
  String prefix
  command <<<
	if [ -f ${deg} ];then
	   if [ -f ${realtion} ];then
	      perl /opt/enrichment.pl -deg ${deg} -all ${all}  -prefix ${dir}/${group}.${prefix} -relation ${realtion}
	   else 
	      cut -f 1 ${deg}|perl -ne '$_=~s/\..*$// if($_!~/MSTRG/);print $_' >${dir}/${group}.${prefix}.deg.list
	      cut -f 1 ${all}|perl -ne '$_=~s/\..*$// if($_!~/MSTRG/);print $_' >${dir}/${group}.${prefix}.all.list
	   fi
	fi
  >>>
  output {
        File dout = "${dir}/${group}.${prefix}.deg.list" 
        File aout = "${dir}/${group}.${prefix}.all.list" 
	File dgo = "${dir}/${group}.${prefix}.deg.GO.list.txt"
	File ago = "${dir}/${group}.${prefix}.all.GO.list.txt"
  }
   runtime{
     docker:"kobas:latest"
     cpu:"1"
     memory:"5GB"
  }
}

task kegg_class {

  File anno
  File all
  File deg
  File kegg
  String group
  File outdir
  String prefix
  File dgo
  File ago

  command {
	if [ -f ${anno} ];then
		perl /opt/KobasAnnoForClassify.pl -KobasAnno ${anno} -KEGGClassify  /opt/KEGG.pathway.classify.list  -od ./
		Rscript /opt/kegg-plot.R  KEGG_PATHWAY.second.class  ${outdir}/${group}.${prefix}.KEGG_PATHWAY.second.class.png ${outdir}/${group}.${prefix}.KEGG_PATHWAY.second.class.pdf
		csplit ${kegg} /--------------------/  -f tmp
		Rscript /opt/kegg_term_top20.r infile=tmp00 outfile=${outdir}/${group}.${prefix}.kegg_top20_term




		if [ -f ${dgo} ];then
			perl /opt/gene_ontology_graph_v1.2.pl -i ${ago} -i ${dgo} -mark Backup -mark DEG -od ${outdir}  -k ${group}.${prefix}

			line=`cat ${dgo}| wc -l `
			threshold="10"
			if  [ $line -gt $threshold ] 
			then 
				perl /opt/draw_GO_DAG_map.pl -All_GO  ${ago}  -DEG_list ${deg} -Key ${group}.${prefix}  -od ${outdir}
			fi
		else 
			perl /opt/anno2GO.pl -anno ${anno} -od ./ -k ${group} -filedb /opt/go.obo
			perl /opt/anno2GO.pl -anno ${all} -od ./ -k all  -filedb /opt/go.obo
			perl /opt/gene_ontology_graph_v1.2.pl -i all.GO.list.txt -i ${group}.GO.list.txt -mark Backup -mark DEG -od ${outdir}  -k ${group}.${prefix}

			line=`cat ${group}.GO.list.txt | wc -l`
			threshold="10"
			if  [ $line -gt $threshold ] 
			then 
				perl /opt/draw_GO_DAG_map.pl -All_GO  all.GO.list.txt  -DEG_list ${deg} -Key ${group}.${prefix}  -od ${outdir}
			fi
		fi 

	fi
  }
  output {
        File result = "${outdir}"

  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"2GB"
  
  }
}


