task hisat2 {

  File Ref
  File R1
  File R2
  String sampleName
  command {
	 hisat2  -p 16  -x  ${Ref} -1 ${R1} -2 ${R2}  -S ${sampleName}.sam
  }
  output {
        File sam = "${sampleName}.sam"
  }
  runtime{
     docker:"mrna:latest"
     cpu:"8"
     memory:"60GB"
  }
}

task samtobam {

  File sam

  command {
	 /bio/samtools view --threads 20 -bS ${sam} > ${sam}.bam
  }
  output {
        File bam = "${sam}.bam"
  }
  runtime{
     docker:"biotk:latest"
     cpu:"10"
     memory:"10GB"
  }
}

task sortbam {

  File bam

  command {
	 /bio/samtools sort -@ 20 ${bam} > ${bam}.sort.bam
  }
  output {
        File out = "${bam}.sort.bam"
  }
  runtime{
     docker:"biotk:latest"
     cpu:"10"
     memory:"20GB"
  }
}

task star {

  File Ref
  File R1
  File R2
  String sampleName
  command {
	 STAR --genomeDir ${Ref}  --readFilesIn ${R1}  ${R2}  --runThreadN 16 --twopassMode Basic  --outReadsUnmapped None --chimSegmentMin 12 --chimJunctionOverhangMin 12 --alignSJDBoverhangMin 10 --alignMatesGapMax 100000  --alignIntronMax 100000  --chimSegmentReadGapMax 3  --alignSJstitchMismatchNmax 5 -1 5 5   --outSAMstrandField intronMotif   --outFileNamePrefix ${sampleName}.
  }
  output {
        File sam = "${sampleName}.Aligned.out.sam"
        File junction = "${sampleName}.Chimeric.out.junction"
  }
  runtime{
     docker:"star:latest"
     cpu:"8"
     memory:"80GB"
  }
}

task bam_stat {

  Array [File] bam

  command {
	perl /opt/bam2map_stat.pl -bam ${sep=',' bam} -od  ./
  }
  output {
        File result ="mappedStat.xls"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"30GB"
  }
}

task depth {

  File bam
  File fai
  String sampleName
  String strand
  command {
	bedtools genomecov -ibam ${bam} -g ${fai} -d  -strand ${strand}  |awk '$3!=0' >${sampleName}.sort.bam.depth
  }
  output {
        File depth = "${sampleName}.sort.bam.depth"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"4"
     memory:"30GB"
  }
}

task ReadDensity {

  File plus
  File minus
  String sampleName
  command {
	perl /opt/plotReadDensity2.pl -i ${plus}:${minus} -o ./  -k ${sampleName}
  }
  output {
        File png = "${sampleName}.coverage.png"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"4"
     memory:"30GB"
  }
}

task map_region {

  File bam
  String sampleName
  File gtf 
  command {
	samtools depth ${bam} >${sampleName}.depth
	perl /opt/get_percent_of_exon_intro_inter_by_gff_v1.4.pl -gtf ${gtf} -i ${sampleName}.depth -od ./ -index ${sampleName} 
  }
  output {
        File png = "${sampleName}.type.png"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"30GB"
  }
}

task randcheck {

  File bam
  String sampleName
  File gtf

  command {
	perl  /opt/trans_randcheck.pl  -i  ${bam}  -gtf ${gtf} -index ${sampleName} -od ./ 
  }
  output {
        File png = "${sampleName}.randcheck.png"
        File pdf = "${sampleName}.randcheck.pdf"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"4"
     memory:"30GB"
  }
}

