import "qc.wdl" as qctool
import "hisat2.wdl" as aligntool
import "stringtie.wdl" as expresstool
import "different.wdl" as difftool
import "Splice_event.wdl" as splicetool
import "star_fusion.wdl" as fusiontool
import "gatk.wdl" as snptool
import "enrichment.wdl" as enrichtool

workflow RNAflow {

  Array[Array[File]] readPairs
  File Ref 
  File? ctat 
  File? realtion
  File? info
  File fa
  File gtf
  File fai
  File dict
  File kobasdb
  Array[String] vs
  Float logfc
  Float fdr
  Float bcv
  String idtype
  String species
  String enrich


  scatter (sample in readPairs) {
    call qctool.name   {
      input: sampleName=sample[0]
    }
    call qctool.fastp {
      input: Fq1=sample[1],Fq2=sample[2],sampleName=name.value
    }
    call aligntool.star  {
      input: R1=fastp.R1,R2=fastp.R2,Ref=Ref,sampleName=name.value
    }
    call aligntool.samtobam  {
      input: sam=star.sam
    }  
    call aligntool.sortbam  {
      input: bam=samtobam.bam
    }    
    #call aligntool.depth as plus {
    #  input: bam=sortbam.out, fai=fai,sampleName=name.value,strand='+'
    #}    
    #call aligntool.depth as minus {
    #  input: bam=sortbam.out, fai=fai,sampleName=name.value,strand='-'
    #} 
    #call aligntool.ReadDensity {
    #  input: plus=plus.depth, minus=minus.depth,sampleName=name.value
    #} 
    call aligntool.map_region {
      input:bam=sortbam.out, gtf=gtf,sampleName=name.value
    } 
    #call aligntool.randcheck {
    #  input:bam=sortbam.out, gtf=gtf,sampleName=name.value
    #} 
    call expresstool.assembly  {
      input: bam=sortbam.out,gtf=gtf,sampleName=name.value
    }

    call fusiontool.fusion  {
	input: ctat=select_first([ctat,'nopath']),junction=star.junction,sampleName=name.value
     }
    call snptool.add  {
      input: sam=star.sam,sampleName=name.value
    }
    call snptool.MarkDuplicates  {
      input: bam=add.bam,sampleName=name.value
    }
    call snptool.SplitNCigarReads {
      input: bam=MarkDuplicates.output_bam,
      bam_index = MarkDuplicates.output_bam_index,
      sampleName=name.value,ref_fasta=fa,ref_fasta_index=fai,ref_dict=dict
    }
    call snptool.HaplotypeCaller {
      input: bam=SplitNCigarReads.output_bam,
      bam_index = SplitNCigarReads.output_bam_index,
      sampleName=name.value,ref_fasta=fa,ref_fasta_index=fai,ref_dict=dict
    }
    call snptool.VariantFiltration {
      input: input_vcf=HaplotypeCaller.output_gvcf,
      input_vcf_index = HaplotypeCaller.output_gvcf_index,
      sampleName=name.value,ref_fasta=fa,ref_fasta_index=fai,ref_dict=dict
    }
    call snptool.bgzip {
      input: input_vcf=VariantFiltration.output_vcf,sampleName=name.value
    }
    call snptool.tbi {
      input: input_vcf=bgzip.output_vcf
    }
 }
 # call qctool.qc_stat  {
  #    input: json=fastp.json,qc_result="QC_Result"
  #}
  call aligntool.bam_stat  {
      input: bam=sortbam.out
  }


  call expresstool.mergegtf  {
      input: gtfs=assembly.out,gtf=gtf,name="AllSample"
  }
  call expresstool.new_trans  {
      input: fa=fa,gtf=mergegtf.out,dir="new_transcript"
  }

  call expresstool.structure_optimize  {
      input: new_gtf=new_trans.finalgtf,old_gtf=gtf
  }

  call qctool.mkdir as ballgowndir {
      input: sampleName="ballgown"
  }
  
  Array[File] sortbams =sortbam.out
  
  scatter (bam in sortbams) {
	  call expresstool.extract {
	      input: bam=bam
	  }  
	  call expresstool.express  {
	      input: bam=bam,gtf=new_trans.finalgtf,dir=ballgowndir.path,sampleName=extract.value
	  }
	  call splicetool.index  {
	      input: bam=bam
	  }
  }
  
  call expresstool.fpkm_stat  {
      input: tab=express.tab
  }
  call expresstool.cor_plot  {
      input: fpkm=fpkm_stat.xls
  }

  call expresstool.exptocsv  {
      input: dir=ballgowndir.path,new=express.cov
  }

  scatter (group in vs) {
	  call qctool.mkdir as diffdir {
	      input: sampleName=group
	  }
	  call difftool.ballgown  {
	      input: outdir=diffdir.path,dir=ballgowndir.path,vs=group,logfc=logfc,fdr=fdr,new=express.cov
	  }
	  call difftool.deseq2  {
	      input: outdir=diffdir.path,count=exptocsv.count,vs=group,logfc=logfc,fdr=fdr
	  }
	  call difftool.edger  {
	      input: outdir=diffdir.path,count=exptocsv.count,vs=group,logfc=logfc,fdr=fdr
	  }
	  call difftool.volcano as bvol  {
	      input: outdir=diffdir.path,all=ballgown.diff,vs=group,logfc=logfc,fdr=fdr
	  }
	  call difftool.volcano as dvol  {
	      input: outdir=diffdir.path,all=deseq2.diff,vs=group,logfc=logfc,fdr=fdr
	  }
	  call difftool.volcano as evol  {
	      input: outdir=diffdir.path,all=edger.diff,vs=group,logfc=logfc,fdr=fdr
	  }
	  call enrichtool.list as  blist  {
	      input: deg=ballgown.deg,all=ballgown.diff,realtion=select_first([realtion,'nopath']),group=group,dir=diffdir.path,prefix="ballgown"
	  }
	  call enrichtool.list as  dlist  {
	      input: deg=deseq2.deg,all=deseq2.diff,realtion=select_first([realtion,'nopath']),group=group,dir=diffdir.path,prefix="DESeq2"
	  }
	  call enrichtool.list as  elist  {
	      input: deg=edger.deg,all=edger.diff,realtion=select_first([realtion,'nopath']),group=group,dir=diffdir.path,prefix="edgeR"
	  }
	  call difftool.fpkm_exact as  bexact  {
	      input: deg=ballgown.deg,group=group,fpkm=fpkm_stat.xls
	  }
	  call difftool.fpkm_exact as  dexact  {
	      input: deg=deseq2.deg,group=group,fpkm=fpkm_stat.xls
	  }
	  call difftool.fpkm_exact as  eexact  {
	      input: deg=edger.deg,group=group,fpkm=fpkm_stat.xls
	  }
	  call difftool.heatmap_plot as  bheatmap  {
	      input: outdir=diffdir.path,fpkm=bexact.out,group=group,prefix="ballgown"
	  }
	  call difftool.heatmap_plot as  dheatmap  {
	      input: outdir=diffdir.path,fpkm=dexact.out,group=group,prefix="DESeq2"
	  }
	  call difftool.heatmap_plot as  eheatmap  {
	      input: outdir=diffdir.path,fpkm=eexact.out,group=group,prefix="edgeR"
	  }

	  call enrichtool.annotate as  bannotate  {
	      input: deg=blist.dout,all=blist.aout,idtype=idtype,species=species,kobasdb=kobasdb,enrich=enrich
	  }
	  call enrichtool.annotate as  dannotate  {
	      input: deg=dlist.dout,all=dlist.aout,idtype=idtype,species=species,kobasdb=kobasdb,enrich=enrich
	  }
	  call enrichtool.annotate as  eannotate  {
	      input: deg=elist.dout,all=elist.aout,idtype=idtype,species=species,kobasdb=kobasdb,enrich=enrich
	  }

	  call enrichtool.kegg_class as  bclass  {
	      input: outdir=diffdir.path,anno=bannotate.danno,all=bannotate.eanno,group=group,prefix="ballgown",deg=blist.dout,kegg=bannotate.kegg,dgo=blist.dgo,ago=blist.ago
	  }
	  call enrichtool.kegg_class as  dclass  {
	      input: outdir=diffdir.path,anno=dannotate.danno,all=dannotate.eanno,group=group,prefix="DESeq2",deg=dlist.dout,kegg=dannotate.kegg,dgo=dlist.dgo,ago=dlist.ago
	  }
	  call enrichtool.kegg_class as  eclass  {
	      input: outdir=diffdir.path,anno=eannotate.danno,all=eannotate.eanno,group=group,prefix="edgeR",deg=elist.dout,kegg=eannotate.kegg,dgo=elist.dgo,ago=elist.ago
	  }

 }

  call qctool.mkdir as splicedir {
      input: sampleName="Splice_event"
  }
  call splicetool.sgseq {
      input: name=extract.value,bam=sortbams,dir=splicedir.path,gtf=gtf
  }

  call snptool.vcfmerge {
      input: input_vcfs=bgzip.output_vcf, input_vcfs_indexes=tbi.output_tbi
  }
  call snptool.snp_stat  {
      input: vcf=vcfmerge.output_vcf
  }
  call snptool.snp_anno  {
      input: vcf=vcfmerge.output_vcf,gtf=gtf,fa=fa
  }
 }
