task assembly {

  File bam
  File gtf
  String sampleName

  command {
	 stringtie  ${bam} -o  ${sampleName}.gtf -G  ${gtf} -C ${sampleName}.cov  -p 12 -A  ${sampleName}.tab 
  }
  output {
        File out = "${sampleName}.gtf" 
  }
  runtime{
     docker:"mrna:latest"
     cpu:"1"
     memory:"5GB"
  }
}
task mergegtf {

  Array[File] gtfs
  File gtf
  String name

  command {
	 stringtie --merge  -o  ${name}_merge.gtf -G  ${gtf}    ${sep=' ' gtfs}  
  }
  output {
        File out = "${name}_merge.gtf" 
  }
  runtime{
     docker:"mrna:latest"
     cpu:"1"
     memory:"5GB"
  }
}

task extract {

  File bam

  command {
	basename ${bam}|cut -d '.' -f 1 >tmp.txt
  }
  output {
        String value =read_string("tmp.txt")
  }
}

task express {

  File bam
  File gtf
  String sampleName
  File dir

  command {
	 stringtie  ${bam} -G  ${gtf} -C ${sampleName}.express.cov -b  ${dir}/${sampleName} -e  -p 6 -A  ${sampleName}.express.tab -o ${dir}/${sampleName}/${sampleName}.express.gtf
  }
  output {
        File ballgown = "${dir}/${sampleName}" 
	File cov="${sampleName}.express.cov"
	File tab="${sampleName}.express.tab"
  }
  runtime{
     docker:"mrna:latest"
     cpu:"1"
     memory:"5GB"
  }
}

task exptocsv {

  File dir
  Array[File]  new

  command {
	 prepDE.py -i ${dir}  -g  ${dir}/gene_count_matrix.csv  -t  ${dir}/transcript_count_matrix.csv
  }
  output {
        File count = "${dir}/gene_count_matrix.csv" 
  }
  runtime{
     docker:"mrna:latest"
     cpu:"1"
     memory:"5GB"
  }
}
task new_trans {

  File fa
  String dir
  File gtf
  command {
	 perl  /Bio/new_transcript.pl -fa  ${fa} -od  ${dir}  -gtf  ${gtf}  
  }
  output {
        File  trans= "${dir}/new.final.transcript.fa" 
        File  newgtf= "${dir}/new.trancript.final.gtf" 
        File  finalgtf= "${dir}/final.all.transcript.gtf" 
  }
  runtime{
     docker:"emboss:latest"
     cpu:"5"
     memory:"10GB"
  }
}

task structure_optimize {

  File new_gtf
  File old_gtf 
  command {
	perl /opt/gene_structure_optimize.pl ${new_gtf}  ${old_gtf} ./ 
  }
  output {
        File xls = "geneStructure.optimize.xls"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"1GB"
  }
}

task fpkm_stat {

  Array[File] tab
  command {
	perl /opt/abstract_fpkm.pl -inputs  ${sep="," tab}  -out gene_fpkm_matrix.xls
	Rscript /opt/fpkm_density_plot_func.r  gene_fpkm_matrix.xls ./
  }
  output {
        File xls = "gene_fpkm_matrix.xls"
        File png = "all.fpkm_box.png"
  }
  runtime{
     docker:"r_perl:latest"
     cpu:"1"
     memory:"1GB"
  }
}

task cor_plot {

  File fpkm 
  command {
        Rscript  /opt/heatmap_cor.r --infile ${fpkm} --prefix sample --cellnote TRUE
  }
  output {
        File dir = "sample.cor_files"
        File html = "sample.cor.html"
  }
  runtime{
     docker:"heatmap:latest"
     cpu:"1"
     memory:"2GB"
  }
}